<?php
// $Id: classroom.access.inc,v 1.8 2009/09/12 13:51:48 osoh Exp $

/**
 * Determines if the node can be accessed.
 *
 * @param $nid
 *   Item nid. 
 * @param $role
 *   Classroom role. What the user must be to access to that node.
 * @param $type
 *   Type to check against the node.
 */

function classroom_check_access( $nid, $role = 'registered', $type = null) {
  global $user;
  $node = node_load(array('nid' => $nid));
  if ($node === NULL) {
    return false;
  }

  if (($type !== NULL) && (_classroom_is_type($node->type, $type) == false)) {
    return false;
  }
 
  $course = classroom_get_course_for($node);

  if (classroom_is_course_owner($user, $course)||
      user_access('administer classroom')) {
    return true;
  }

  switch($role) {
  case 'registered':
    return classroom_is_user_active($user, $course);
    break;
  case 'teacher':
    return classroom_is_active_as_teacher($user, $course);
    break;
  case 'student':
    return classroom_is_active_as_student($user, $course);
    break;
  default:
    return false;
    break;
  }
}

/**
 * Get students' permissions.
 *
 * @return
 *   Returns an array containing the added permissions if they were added.
 *   It returns false in case of error.
 */
function classroom_perms() {
  // Student's perms
  $student_perms = array('access classroom');
  $response_types = _classroom_get_types_for('response');
  foreach ($response_types as $type) {
    array_push($student_perms, "create $type->type content");
    array_push($student_perms, "edit own $type->type content");
    array_push($student_perms, "delete own $type->type content");
  }

  // Teachers' perms
  $teacher_perms = array('access classroom');
  $assignment_types = _classroom_get_types_for('assignment');
  foreach ($assignment_types as $type) {
    array_push($teacher_perms, "create $type->type content");
    array_push($teacher_perms, "edit own $type->type content");
    array_push($teacher_perms, "delete own $type->type content");
    array_push($teacher_perms, "edit any $type->type content");
    array_push($teacher_perms, "delete any $type->type content");
  }

  $response_types = _classroom_get_types_for('response');
  foreach ($response_types as $type) {
    array_push($teacher_perms, "edit any $type->type content");
  }

  return array('student' => $student_perms, 'teacher' => $teacher_perms);
}

/**
 * Add permissions to a role.
 *
 * @param $rid
 *   Role ID.
 * @param $perms
 *   Permissions to add.
 * @return
 *   Returns an array containing the added permissions if they were added.
 *   It returns false in case of error.
 */
function classroom_add_perms($rid, $perms = array()) {
  $result = db_query("SELECT * FROM {permission} WHERE rid = %d", $rid);
  $old_perms_obj = db_fetch_object($result);

  if (!$result) {
    return FALSE;
  }

  // Sanitize $perms array.
  // TODO: it must check that permissions are valid.
  $perms = array_unique($perms);

  if ($old_perms_obj) {
    $old_perms = explode(', ', $old_perms_obj->perm);
    $added_perms = array_diff($perms, $old_perms);
    $new_perms = array_merge($old_perms, $added_perms);
    $new_perms_string = implode(', ', $new_perms);
    $result = db_query("UPDATE {permission} SET rid = %d, perm = '%s' WHERE pid = %d", $rid, $new_perms_string, (int) $old_perms_obj->pid);
  } else {
    $result = db_query("INSERT INTO {permission} (rid, perm) VALUES (%d, '%s')",
      $rid, implode(', ', $perms));
    $added_perms = $perms;
  }
  return $added_perms;
}

/**
 * Remove permissions to a role.
 *
 * @param $rid
 *   Role ID.
 * @param $perms
 *   Permissions to remove.
 * @return
 *   Returns an array containing the removed permissions if they were removed.
 *   It returns false in case of error.
 */
function classroom_remove_perms($rid, $perms = array()) {
  $result = db_query("SELECT * FROM {permission} WHERE rid = %d", $rid);
  $old_perms_obj = db_fetch_object($result);

  if (!$result) {
    return FALSE;
  }

  // Sanitize $perms array.
  // TODO: it must check that permissions are valid.
  $perms = array_unique($perms);

  if ($old_perms_obj) {
    $old_perms = explode(', ', $old_perms_obj->perm);
    $removed_perms = array_intersect($perms, $old_perms);
    $new_perms = array_diff($old_perms, $removed_perms);
    $new_perms_string = implode(', ', $new_perms);
    $result = db_query("UPDATE {permission} SET rid = %d, perm = '%s' WHERE pid = %d", $rid, $new_perms_string, (int) $old_perms_obj->pid);
  } else {
    $removed_perms = array();
  }
  return $removed_perms;
}
